###############################################################################
###############################################################################
#####                            Ventana.tcl
###############################################################################
##### Creates the window which will provide some information to the user 
##### while a file is being downloaded.
###############################################################################
##### Copyright 1999 Andrs Garca Garca - ornalux@redestb.es
##### The contents of this file are distributed under the terms of the GPL
###############################################################################

namespace eval Ventana {

source [file join "$dirGetleft(scripts)" Rizo.tcl]

###############################################################################
# StopDownload
#    Stops downloading
###############################################################################
proc StopDownload {} {

    set ::opciones(stopFile) 1
    set Rizo::informe(parar) 1
    Rizo::Lector $::Ventana::Rizo::pipe stopNow

    return
}

###############################################################################
# CancelDownloading
#    This procedure is invoked from the downloading window when you click on
#    the 'cancel' button or you want to close the window
###############################################################################
proc CancelDownloading {} {
    global labelTitles labelMessages
    global getleftState
    variable afterId

    set what [tk_messageBox -type yesno -icon question \
            -title $labelTitles(confirm) -message $labelMessages(stopDownload)]

    if {$what=="no"} {
        return
    }
    catch {after cancel $afterId}
    StopDownload
    catch {destroy .bajando}

    set getleftState(downloading) 0

    return
}

###############################################################################
# RecortarCadena
#    Recorta el nombre del fichero o del enlace para que cuadre con el espacio
#    que hay para escribirlos.
#
# Parmetros
#    cadena: cadena con el nombre a recortar
#    long: longitud en la que tiene que quedar
#
# Devuelve:
#    La cadena recortada
###############################################################################
proc RecortarCadena {cadena long} {

    set longitud_cadena [string length $cadena]
    if {$longitud_cadena>$long} {
        set index  [expr round($long/2)-2]
        set fin    [expr $longitud_cadena-$index]
        set primer [string range $cadena 0 $index]
        set segun  [string range $cadena $fin end]
        append tmp $primer "..." $segun
        return $tmp
    } else {
        return $cadena
    }
}

###############################################################################
# FileStrings
#    Controls the strings that will be put in the downloading window, to
#    show the name and directory of the file being processed or downloaded
#
# Parameter:
#    File to be downloaded or preprocessed
###############################################################################
proc FileStrings {fichero} {
    global directorio_base direccion
    variable window
    variable labelWidth

    regexp "(?:$directorio_base)(.*)" [file rootname $fichero] nada tmp
    regexp {(.*)(?:/)(.*)} $tmp nada tmpDir tmpFile

    $window(url)  configure -text [RecortarCadena $direccion(www) $labelWidth(url)]
    $window(dir)  configure -text [RecortarCadena $tmpDir  $labelWidth(dir)]
    $window(file) configure -text [RecortarCadena $tmpFile $labelWidth(dir)]

    return
}

###############################################################################
# CrearVentana
#    Creates the window, in which some info about the downloading will be
#    displayed.
###############################################################################
proc CrearVentana {} {
    global Ventana::Rizo::meta Ventana::Rizo::mensaje_error
    global file link mother coord labelButtons labelDialogs
    global tcl_platform
    variable window
    variable labelWidth

    set bajando [toplevel .bajando]

    fijar_coordenadas
    wm geometry $bajando 290x155+[expr $coord(x)+200]+[expr $coord(y)+75]
    wm resizable $bajando 0 0

# This is a kludge that will have to be mended
    if [regexp {Windows} $tcl_platform(os)] {
	    set labelWidth(url) 46
        set labelWidth(dir) 40
    } else {
	    set labelWidth(url) 37
        set labelWidth(dir) 31
    }

    set marco     [frame $bajando.marco]
    set marco_in  [frame $bajando.marco.marco_interno]
    set marco_url [frame $marco_in.url]
    set marco_dir [frame $marco_in.dir]
    set marco_fil [frame $marco_in.fil]
    set marco_que [frame $marco_in.que]
    set marco_bar [frame $marco_in.bar]
    set botones   [frame $bajando.botones]

    set etq_url  [label $marco_url.etq_url  -text "$labelDialogs(url) "]
    set etq_dir  [label $marco_dir.etq_dir  -text "$labelDialogs(dir): "]
    set etq_file [label $marco_fil.etq_file -text "$labelDialogs(file) "]

    set Ventana::Rizo::informe(cual) "$labelDialogs(left) "
    set etq_que   [label $marco_que.etq_que   -textvariable Ventana::Rizo::informe(cual)]
    set etq_url2  [label $marco_url.etq_url2  -bd 2 -relief sunken -width 45 -anchor w]
    set etq_dir2  [label $marco_dir.etq_dir2  -width 45 -anchor w]
    set etq_file2 [label $marco_fil.etq_file2 -width 45 -anchor w]
    set etq_que2  [label $marco_que.etq_que2  -textvariable Ventana::Rizo::informe(queda) -width 30 -anchor w]

    set barra1 [canvas $marco_bar.lienzo1 -width 275 -height 12 -bg white -bd 2 -relief sunken -highlightthickness 0]
    set barra2 [canvas $barra1.lienzo2    -bg blue -highlightthickness 0]

    set window(top)  .bajando
    set window(url)  $etq_url2
    set window(dir)  $etq_dir2
    set window(file) $etq_file2
    set window(left) $etq_que2
    set window(bar1) $barra1
    set window(bar2) $barra2

    set pause [button $botones.pause -text $labelButtons(pause)    \
        -command ::Ventana::PauseDownloading  -width 8]
    set cancelar [button $botones.cancelar -text $labelButtons(cancel) \
        -command ::Ventana::CancelDownloading -width 8]

    pack $marco     -side top    -fill x -expand true
    pack $botones   -side bottom -fill x -expand true -padx 6
    pack $marco_in  -padx 10 -pady 10
    pack $marco_url $marco_in $marco_dir $marco_fil $marco_que $marco_bar -fill x -expand true
    pack $etq_url   $etq_url2  -side left
    pack $etq_dir   $etq_dir2  -side left
    pack $etq_file  $etq_file2 -side left
    pack $etq_que   $etq_que2  -side left
    pack $barra1

    pack $cancelar $pause -side right -padx 3 -pady 5

    bind $pause  <Return> "
        $pause invoke
    "
    bind $cancelar <Return> "
        $cancelar invoke
    "
    wm protocol $bajando WM_DELETE_WINDOW ::Ventana::CancelDownloading

    return
}

###############################################################################
# ShowWindow
#   Is needed, maps the downloading window, and sets its title bar
#
# Parameter
#   which: either 'down' or 'process', depending on the action taking place
###############################################################################
proc ShowWindow {which} {
    global labelTitles

    if {![winfo exists .bajando]} {
        CrearVentana
    } else {
        set state [wm state .bajando]
        if {($state=="withdrawn")&&($which=="down")} {
            wm deiconify .bajando
        }
    }
    if {$which=="down"} {
        wm title .bajando $labelTitles(download)
    } else {
        wm title .bajando $labelTitles(process)
    }

    return
}

###############################################################################
# Bajando_archivo
#   Creates a nice window to show info about how the downloading goes
#
# Parameters:
#    fichero: file in which the link will be saved.
#    enlace:  file url.
#    madre:   referer page for the link
###############################################################################
proc Bajando_archivo {fichero enlace madre} {
    global file link mother directorio_base
    global getleftState
    global labelTitles
    variable window
    variable labelWidth

    if {$getleftState(downloading)==0} {
        return
    }

    set file   $fichero
    set link   $enlace
    set mother $madre

    ShowWindow down
    FileStrings $fichero

    set Ventana::Rizo::informe(fin)          0
    set Ventana::Rizo::informe(largo)        1
    set Ventana::Rizo::informe(queda)        ""
    set Ventana::Rizo::informe(pause)        0

    if {([file exists $fichero])&&($Rizo::meta(versionServer)>=1.1)} {
        if {![catch {set meta(totalBytes)}]} {
            set size [file size $fichero]
            if {$size>=$meta(totalBytes)} {
                file delete $fichero
            }
        } else {
            file delete $fichero
        }
        Ventana::Rizo::Reanudar $fichero $enlace $madre
    } else {
        Ventana::Rizo::Datos $fichero $enlace $madre
    }

    Barra_actualizar $window(bar1)

    return
}

###############################################################################
# Bajando_encabezamiento
#    Gets the headers of the url from the server.
#
# Parameters:
#    enlace: url to download
#    madre:  referer page of the link
###############################################################################
proc Bajando_encabezamiento {enlace madre} {

    set Ventana::Rizo::informe(fin) 0
    Ventana::Rizo::Cabeza $enlace $madre
    tkwait variable Ventana::Rizo::informe(fin)

    return
}

###############################################################################
# PauseDownloading
#    Takes control when you click on the 'Pause' button
###############################################################################
proc PauseDownloading {} {
    global file link mother
    global labelButtons
    variable window

    set Ventana::Rizo::informe(pause) 1
    set boton ".bajando.botones.pause"
    $boton configure -text $labelButtons(resume) -command "
        set Ventana::Rizo::informe(pause) 0
        .bajando.botones.pause configure -text $labelButtons(pause) \
                -command ::Ventana::PauseDownloading
        place forget $window(bar2)
        Ventana::Rizo::Reanudar \"$file\" $link \"$mother\"
        Ventana::Barra_actualizar $window(bar1)
    "
    return
}

###############################################################################
# Barra_actualizar
#    Actualiza la barra de progreso.
#
# Parmetros:
#    barra: path de la barra de progreso
###############################################################################
proc Barra_actualizar {barra} {
    global Ventana::Rizo::informe
    global Ventana::Rizo::curlError
    global Ventana::Rizo::meta
    global labelDialogs
    global file link mother opciones
    variable window
    variable afterId
    variable uno
    variable dos

    if [string match $informe(porcentaje) ""] return
    set hecho [expr 2.66 * $informe(porcentaje)]

    if {($informe(largo)==0)} {
        set informe(cual) "$labelDialogs(got) "
    } else {
        set informe(cual) "$labelDialogs(left) "
    }

    place $barra.lienzo2 -in $barra -x 2 -y 2 -bordermode inside -width $hecho -height 12
    if [info exists uno] {
        $barra delete $uno
        $barra.lienzo2 delete $dos
    }
    set uno [$barra create         text 137.5 9 -text $informe(porcentaje)% -fill black -anchor c]
    set dos [$barra.lienzo2 create text 134.5 7 -text $informe(porcentaje)% -fill white -anchor c]

    if {$informe(pause)==0} {
        if {($informe(fin)==0) && ($informe(parar)!=1)} {
            set afterId [after 1000 Ventana::Barra_actualizar $barra]
        } else {
            if {$meta(totalBytes)!=0} {
                if {($curlError=="")||($curlError==0)} {
                    if {![catch [list file stat "$file" fileData] ]} {
                        if {$fileData(size)!=$meta(totalBytes)} {
                            if {($opciones(resume)==1)&&($informe(parar)!=1)} {
                                $barra create rect 0 0 280 15 -fill white
                                Bajando_archivo $file $link $mother
                                return
                            } else {
                                catch {error "Fichero Incompleto" GETLEFT \
                                    "Getleft \"$labelDialogs(fileIncom)\" 001"}
                            }
                        }
                    }
                }
            }
            place $barra.lienzo2 -in $barra -x 2 -y 2 -bordermode inside -width 266 -height 12
            update
            after 75
            place forget $barra.lienzo2
            $barra delete $uno
            $window(dir) configure -text ""
            $window(file) configure -text ""
            $window(left) configure -text ""
            incr informe(nextFile)
        }
    }
    return
}

}
